@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -11,6 +11,18 @@
 //-----------------------------------------------------------------
 // Import/Export PWScore member functions
 //-----------------------------------------------------------------
+#if !defined(_WIN32)
+#include <wx/wxprec.h>
+
+#ifndef WX_PRECOMP
+#include <wx/wx.h>
+#endif
+
+#ifdef __WXMSW__
+#include <wx/msw/msvcrt.h>
+#endif
+#endif
+
 #include "PWScore.h"
 #include "core.h"
 #include "PWSprefs.h"
@@ -34,6 +46,8 @@
 #include "XML/MSXML/MFileXMLProcessor.h"
 #elif USE_XML_LIBRARY == XERCES
 #include "XML/Xerces/XFileXMLProcessor.h"
+#else
+#include "XML/Pugi/PFileXMLProcessor.h"
 #endif
 
 #include <fstream> // for WritePlaintextFile
@@ -723,7 +737,7 @@ int PWScore::WriteXMLFile(const StringX &filename,
  return numXMLErrors == 0 ? SUCCESS : OK_WITH_ERRORS;
 }
 
-#if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
+#if (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
 // Don't support importing XML on non-Windows platforms using Microsoft XML libraries
 int PWScore::ImportXMLFile(const stringT & /*ImportedPrefix*/,
                            const stringT & /*strXMLFileName*/,
@@ -741,7 +755,11 @@ int PWScore::ImportXMLFile(const stringT & /*ImportedPrefix*/,
 }
 #else
 int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLFileName,
-                           const stringT &strXSDFileName, const bool &bImportPSWDsOnly,
+                           const stringT &
+#if defined(USE_XML_LIBRARY)
+                                           strXSDFileName
+#endif
+                                                          , const bool &bImportPSWDsOnly,
                            stringT &strXMLErrors, stringT &strSkippedList,
                            stringT &strPWHErrorList, stringT &strRenameList,
                            int &numValidated, int &numImported, int &numSkipped,
@@ -758,15 +776,29 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
   MFileXMLProcessor iXML(this, &Possible_Aliases, &Possible_Shortcuts, pmulticmds, &rpt);
 #elif USE_XML_LIBRARY == XERCES
   XFileXMLProcessor iXML(this, &Possible_Aliases, &Possible_Shortcuts, pmulticmds, &rpt);
+#else
+  PFileXMLProcessor iXML(this, &Possible_Aliases, &Possible_Shortcuts, pmulticmds, &rpt);
 #endif
 
   bool status, validation;
 
   strXMLErrors = strPWHErrorList = strRenameList = _T("");
+    
+#if !defined(USE_XML_LIBRARY)
+  status = iXML.ReadXML(_T(""), strXMLFileName);
+  strXMLErrors = iXML.getXMLErrors();
+  if (!status) {
+    return XML_FAILED_VALIDATION;
+  }
+#endif
 
   validation = true;
+#if !defined(USE_XML_LIBRARY)
+  status = iXML.Process(validation, ImportedPrefix, bImportPSWDsOnly);
+#else
   status = iXML.Process(validation, ImportedPrefix, strXMLFileName,
                         strXSDFileName, bImportPSWDsOnly);
+#endif
   strXMLErrors = iXML.getXMLErrors();
   if (!status) {
     return XML_FAILED_VALIDATION;
@@ -775,8 +807,12 @@ int PWScore::ImportXMLFile(const stringT &ImportedPrefix, const stringT &strXMLF
   numValidated = iXML.getNumEntriesValidated();
 
   validation = false;
+#if !defined(USE_XML_LIBRARY)
+  status = iXML.Process(validation, ImportedPrefix, bImportPSWDsOnly);
+#else
   status = iXML.Process(validation, ImportedPrefix, strXMLFileName,
                         strXSDFileName, bImportPSWDsOnly);
+#endif
 
   numImported = iXML.getNumEntriesImported();
   numSkipped = iXML.getNumEntriesSkipped();
@@ -865,7 +901,8 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
 
   CItemData ci_temp;
   vector<string> vs_Header;
-  stringT cs_hdr = EXPORTHEADER;
+  CItemData::FieldBits all(~0UL);
+  StringX cs_hdr = BuildHeader(all, true); // Use same language as in writing
 
   // Parse the header
   const unsigned char *hdr;
@@ -875,7 +912,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
   const char pTab[] = "\t";
   char pSeps[] = " ";
 
-  // Order of fields determined in CItemData::GetPlaintext()
+  // Order of fields determined in CItemData::GetPlaintext() and must be same as in BuildHeader()
   enum Fields {GROUPTITLE, USER, PASSWORD, URL, AUTOTYPE,
                CTIME, PMTIME, ATIME, XTIME, XTIME_INT, RMTIME,
                POLICY, POLICYNAME, HISTORY, RUNCMD, DCA, SHIFTDCA, EMAIL,
@@ -960,8 +997,6 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
 
   to = 0;
   do {
-    // make sure EXPORTHEADER has correct field names, though we have some resiliency below
-    ASSERT( vs_Header[itoken] == CItemData::EngFieldName(fieldMap[itoken].itemField));
     from = s_header.find_first_not_of(pSeps, to);
     if (from == string::npos)
       break;
@@ -984,6 +1019,12 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
       i_Offset[ fieldMap[itoken].hdrField ] = itoken;
       num_found++;
     }
+    else {
+      StringX sh2;
+      conv.FromUTF8(reinterpret_cast<const unsigned char *>(token.c_str()), token.length(), sh2);
+      Format(cs_error, L"Not found heading %ls", sh2.c_str());
+      rpt.WriteLine(cs_error, false);
+    }
     itoken++;
   } while (to != string::npos);
 
@@ -1017,7 +1058,7 @@ int PWScore::ImportPlaintextFile(const StringX &ImportedPrefix,
         const string &sHdr = vs_Header.at(i);
         StringX sh2;
         conv.FromUTF8(reinterpret_cast<const unsigned char *>(sHdr.c_str()), sHdr.length(), sh2);
-        Format(cs_error, L" %ls,", sh2.c_str());
+        Format(cs_error, L" %ls%c", sh2.c_str(), ((i+1) == NUMFIELDS) ? ' ' : ',');
         rpt.WriteLine(cs_error, false);
       }
     }
