@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -176,7 +176,7 @@ int CItemData::Read(PWSfile *in)
           trashMemory(utf8, utf8Len * sizeof(utf8[0]));
           delete[] utf8;
         }
-        return -numread;
+        return static_cast<int>(-numread);
       } else if (type != END) { // unknown field
         SetUnknownField(type, utf8Len, utf8);
       }
@@ -559,7 +559,7 @@ void CItemData::GetUUID(uuid_array_t &uuid_array, FieldType ft) const
       ASSERT(0);
     }
   if (fiter == m_fields.end()) {
-    pws_os::Trace(_T("CItemData::GetUUID(uuid_array_t) - no UUID found!\n"));
+    // pws_os::Trace(_T("CItemData::GetUUID(uuid_array_t) - no UUID found!\n"));
     memset(uuid_array, 0, length);
   } else {
     CItem::GetField(fiter->second,
@@ -648,6 +648,12 @@ StringX CItemData::GetProtected() const
   return IsProtected() ? StringX(_T("1")) : StringX(_T(""));
 }
 
+bool CItemData::IsDCASet(bool bShift) const
+{
+    auto fiter = m_fields.find(bShift ? SHIFTDCA : DCA);
+    return (fiter != m_fields.end());
+}
+
 int16 CItemData::GetDCA(int16 &iDCA, const bool bShift) const
 {
   auto fiter = m_fields.find(bShift ? SHIFTDCA : DCA);
@@ -1760,6 +1766,10 @@ bool CItemData::Matches(int16 dca, int iFunction, const bool bShift) const
       return iDCA == dca;
     case PWSMatch::MR_ISNOT:
       return iDCA != dca;
+    case PWSMatch::MR_PRESENT:
+      return IsDCASet(bShift);
+    case PWSMatch::MR_NOTPRESENT:
+      return ! IsDCASet(bShift);
     default:
       ASSERT(0);
   }
@@ -1939,8 +1949,9 @@ bool CItemData::DeSerializePlainText(const std::vector<char> &v)
     }
 
 #ifdef PWS_BIG_ENDIAN
-    unsigned char buf[len] = {0};
-
+    unsigned char buf[len];
+    memset(buf, 0, len*sizeof(buf[0]));
+	  
     switch(type) {
       case CTIME:
       case PMTIME:
@@ -2056,13 +2067,13 @@ void CItemData::SetEntryType(EntryType et)
    * we need to move the UUID to the correct field.
    * In other cases we leave the UUID untouched.
    */
-  if (m_entrytype == ET_NORMAL) {
+  if (m_entrytype == ET_NORMAL || m_entrytype == ET_ALIASBASE || m_entrytype == ET_SHORTCUTBASE) {
     if (et == ET_ALIAS || et == ET_SHORTCUT) {
       const CUUID uuid = GetUUID(UUID);
       SetUUID(uuid, et == ET_ALIAS ? ALIASUUID : SHORTCUTUUID);
       m_fields.erase(UUID);
     }
-  } else if (et == ET_NORMAL) {
+  } else if (et == ET_NORMAL || m_entrytype == ET_ALIASBASE || m_entrytype == ET_SHORTCUTBASE) {
     if (m_entrytype == ET_ALIAS || m_entrytype == ET_SHORTCUT) {
       const CUUID uuid = GetUUID(m_entrytype == ET_ALIAS ? ALIASUUID : SHORTCUTUUID);
       SetUUID(uuid, UUID);
@@ -2072,44 +2083,6 @@ void CItemData::SetEntryType(EntryType et)
   m_entrytype = et;
 }
 
-static void push_length(vector<char> &v, uint32 s)
-{
-  v.insert(v.end(),
-    reinterpret_cast<char *>(&s), reinterpret_cast<char *>(&s) + sizeof(uint32));
-}
-
-template< typename T>
-static void push(vector<char> &v, char type, T value)
-{
-  if (value != 0) {
-    v.push_back(type);
-    push_length(v, sizeof(value));
-    v.insert(v.end(),
-             reinterpret_cast<char *>(&value), reinterpret_cast<char *>(&value) + sizeof(value));
-  }
-}
-
-// Overload rather than specialize template function
-// See http://www.gotw.ca/publications/mill17.htm
-static void push(vector<char> &v, char type,
-                 const StringX &str)
-{
-  if (!str.empty()) {
-    CUTF8Conv utf8conv;
-    bool status;
-    const unsigned char *utf8;
-    size_t utf8Len;
-    status = utf8conv.ToUTF8(str, utf8, utf8Len);
-    if (status) {
-      v.push_back(type);
-      push_length(v, static_cast<uint32>(utf8Len));
-      v.insert(v.end(), reinterpret_cast<const char *>(utf8),
-               reinterpret_cast<const char *>(utf8) + utf8Len);
-    } else
-      pws_os::Trace(_T("ItemData.cpp: push(%ls): ToUTF8 failed!\n"), str.c_str());
-  }
-}
-
 void CItemData::SerializePlainText(vector<char> &v,
                                    const CItemData *pcibase)  const
 {
@@ -2148,7 +2121,7 @@ void CItemData::SerializePlainText(vector<char> &v,
     v.insert(v.end(), uuid_array, (uuid_array + sizeof(uuid_array_t)));
   }
 
-  // TODO - Get rid of following [password hack - no longer needed as we have base uuid!
+  // TODO: - Get rid of following [password hack - no longer needed as we have base uuid!
   if (m_entrytype == ET_ALIAS) {
     // I am an alias entry
     tmp = _T("[[") + pcibase->GetGroup() + _T(":") + pcibase->GetTitle() + _T(":") + pcibase->GetUser() + _T("]]");
