@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -49,9 +49,9 @@ static void ParseNotes(StringX &sxNotes,
                        std::vector<StringX> &vsxnotes_lines)
 {
   if (!sxNotes.empty()) {
-    // Use \n and \r to tokenise this line
+    // Use \n to tokenise this line
     StringX::size_type st_start(0), st_end(0);
-    const StringX sxdelim = _T("\r\n");
+    const StringX sxdelim = _T("\n");
     StringX sxline;
     StringX::size_type st_index;
     while (st_end != StringX::npos) {
@@ -559,14 +559,15 @@ StringX PWSAuxParse::GetAutoTypeString(const StringX &sx_in_autotype,
           break;
         }
 
-        // Also copy explicit control characters to output string unchanged.
+        case TCHAR('e'): // escape
+          sxtmp += L'\x1B';
+          break;        // Also copy explicit control characters to output string unchanged.
         case TCHAR('a'): // bell (can't hear it during testing!)
         case TCHAR('v'): // vertical tab
         case TCHAR('f'): // form feed
-        case TCHAR('e'): // escape
         case TCHAR('x'): // hex digits (\xNN)
         // and any others we have forgotten!
-        // '\cC', '\uXXXX', '\OOO', '\<any other character not recognised above>'
+        // '\cC', '\uXXXX', '\OOO', '\<any other character not recognized above>'
         default:
           sxtmp += L'\\';
           sxtmp += curChar;
