@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -27,6 +27,10 @@
 #include "XML/Xerces/XFilterXMLProcessor.h"
 #endif
 
+#if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
+#include "XML/Pugi/PFilterXMLProcessor.h"
+#endif
+
 #define PWS_XML_FILTER_VERSION 1
 
 #include <fstream>
@@ -678,15 +682,51 @@ std::string PWSFilters::GetFilterXMLHeader(const StringX &currentfile,
 }
 
 #if !defined(USE_XML_LIBRARY) || (!defined(_WIN32) && USE_XML_LIBRARY == MSXML)
-// Don't support importing XML from non-Windows using Microsoft XML libraries
-int PWSFilters::ImportFilterXMLFile(const FilterPool, 
-                                    const StringX &, 
-                                    const stringT &, 
-                                    const stringT &,
-                                    stringT &,
-                                    Asker *)
+int PWSFilters::ImportFilterXMLFile(const FilterPool fpool,
+                                    const StringX &strXMLData,
+                                    const stringT &strXMLFileName,
+                                    const stringT & /* strXSDFileName */,
+                                    stringT &strErrors,
+                                    Asker *pAsker, Reporter *pReporter)
 {
-  return PWScore::UNIMPLEMENTED;
+  PFilterXMLProcessor fXML(*this, fpool, pAsker, pReporter);
+  bool status, validation;
+    
+  strErrors = _T("");
+    
+  if (strXMLFileName.empty())
+    status = fXML.ReadXML(strXMLData, _T(""));
+  else
+    status = fXML.ReadXML(_T(""), strXMLFileName);
+      
+  strErrors = fXML.getXMLErrors();
+  if (!status) {
+    return PWScore::XML_FAILED_VALIDATION;
+  }
+    
+  validation = true;
+  if (strXMLFileName.empty())
+    status = fXML.Process(validation);
+  else
+    status = fXML.Process(validation);
+    
+  strErrors = fXML.getXMLErrors();
+  if (!status) {
+    return PWScore::XML_FAILED_VALIDATION;
+  }
+
+  validation = false;
+  if (strXMLFileName.empty())
+    status = fXML.Process(validation);
+  else
+    status = fXML.Process(validation);
+
+  strErrors = fXML.getXMLErrors();
+  if (!status) {
+    return PWScore::XML_FAILED_IMPORT;
+  }
+    
+  return PWScore::SUCCESS;
 }
 #else
 int PWSFilters::ImportFilterXMLFile(const FilterPool fpool,
@@ -694,7 +734,7 @@ int PWSFilters::ImportFilterXMLFile(const FilterPool fpool,
                                     const stringT &strXMLFileName,
                                     const stringT &strXSDFileName,
                                     stringT &strErrors,
-                                    Asker *pAsker)
+                                    Asker *pAsker, Reporter *)
 {
 #if USE_XML_LIBRARY == MSXML
   MFilterXMLProcessor fXML(*this, fpool, pAsker);
@@ -771,6 +811,8 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
       // Note: purpose drop through to standard 'string' processing
       //[[fallthrough]];
     case PWSMatch::MT_STRING:
+      //[[fallthrough]];
+    case PWSMatch::MT_MEDIATYPE:
       if (st_fldata.rule == PWSMatch::MR_PRESENT ||
           st_fldata.rule == PWSMatch::MR_NOTPRESENT)
         Format(cs_criteria, L"%ls", cs_rule.c_str());
@@ -791,7 +833,7 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
         Format(cs_criteria, L"%ls", cs_rule.c_str());
       else
       if (st_fldata.rule == PWSMatch::MR_BETWEEN) {  // Date or Integer only
-        LoadAString(cs_and, IDSC_AND);
+        LoadAString(cs_and, IDSC_AND_SMALL);
         Format(cs_criteria, L"%ls %ls %ls %ls", 
                cs_rule.c_str(), cs1.c_str(), cs_and.c_str(), cs2.c_str());
       } else {
@@ -800,8 +842,8 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
       if (st_fldata.mtype == PWSMatch::MT_DATE &&
           st_fldata.fdatetype == 1 /* Relative */) {
         stringT cs_temp;
-        LoadAString(cs_temp, IDSC_RELATIVE);
-        cs_criteria += cs_temp;
+        LoadAString(cs_temp, IDSC_RELATIVE_SMALL);
+        cs_criteria += _T(" ") + cs_temp;
       }
       if (st_fldata.mtype == PWSMatch::MT_ENTRYSIZE) {
         switch (st_fldata.funit) {
@@ -843,9 +885,6 @@ stringT PWSFilters::GetFilterDescription(const st_FilterRow &st_fldata)
     case PWSMatch::MT_ENTRYSTATUS:
       Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
       break;
-    case PWSMatch::MT_MEDIATYPE:
-      Format(cs_criteria, L"%ls %ls", cs_rule.c_str(), cs1.c_str());
-      break;
     default:
       ASSERT(0);
   }
