@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -379,7 +379,7 @@ class PWSFilters : public std::map<st_Filterkey, st_filters, ltfk> {
                           const StringX &strXMLData,
                           const stringT &strXMLFileName,
                           const stringT &strXSDFileName, stringT &strErrors,
-                          Asker *pAsker);
+                          Asker *pAsker, Reporter *pReporter = nullptr);
 
   static stringT GetFilterDescription(const st_FilterRow &st_fldata);
  private:
