@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -63,7 +63,7 @@ struct st_ValidateResults;
 class PWScore : public Observable, public CommandInterface
 {
 public:
-  enum {
+  enum Status {
     SUCCESS = 0,
     FAILURE = 1,
     USER_DECLINED_SAVE = 2,
@@ -92,6 +92,7 @@ public:
     OPEN_NODB,                                //  24
     MAX_SIZE_EXCEEDED                         //  25
   };
+  static stringT StatusText(int s);
 
   PWScore();
   ~PWScore();
@@ -266,7 +267,7 @@ public:
   void GetAllGroups(std::vector<stringT> &vAllGroups, const bool bIncludeEmptyGroups = true) const;
   // Construct unique title
   StringX GetUniqueTitle(const StringX &group, const StringX &title,
-                         const StringX &user, const int IDS_MESSAGE);
+                         const StringX &user, const int ids_messsage);
   // Get all password policy names
   void GetPolicyNames(std::vector<stringT> &vNames) const;
   bool GetPolicyFromName(const StringX &sxPolicyName, PWPolicy &st_pp) const;
@@ -274,12 +275,12 @@ public:
                              std::map<StringX, StringX> &mapRenamedPolicies,
                              std::vector<StringX> &vs_PoliciesAdded,
                              StringX &sxOtherPolicyName, bool &bUpdated,
-                             const StringX &sxDateTime, const UINT &IDS_MESSAGE);
+                             const StringX &sxDateTime, const UINT &ids_message);
 
   // This routine should only be directly called from XML import
   void MakePolicyUnique(std::map<StringX, StringX> &mapRenamedPolicies,
                         StringX &sxPolicyName, const StringX &sxDateTime,
-                        const UINT IDS_MESSAGE);
+                        const UINT ids_message);
 
   bool GetEntriesUsingNamedPasswordPolicy(const StringX sxPolicyName,
                                           std::vector<st_GroupTitleUser> &ventries);
@@ -292,7 +293,7 @@ public:
   // Adds an st_GroupTitleUser to setGTU, possible modifying title
   // to ensure uniqueness. Returns false if title was modified.
   bool MakeEntryUnique(GTUSet &setGTU, const StringX &group, StringX &title,
-                       const StringX &user, const int IDS_MESSAGE);
+                       const StringX &user, const int ids_message);
   bool GetUniqueGTUValidated() const
   {return m_bUniqueGTUValidated;}
 
@@ -329,7 +330,7 @@ public:
   ItemListConstIter Find(const pws_os::CUUID &entry_uuid) const
   {return m_pwlist.find(entry_uuid);}
 
-  bool ConfirmDelete(const CItemData *pci); // ask user when about to delete a base,
+  bool ConfirmDelete(const CItemData *pci, StringX sxGroup = L""); // ask user when about to delete a base,
   //                                           otherwise just return true
 
   // General routines for aliases and shortcuts
@@ -594,7 +595,8 @@ private:
   void SetBase2ShortcutsMmap(ItemMMap &b2smm) {m_base2shortcuts_mmap = b2smm;}
   
   // Following are private in PWScore, public in CommandInterface:
-  void AddChangedNodes(StringX path);
+  void AddChangedNodes(const StringX &path);
+  void AddChangedEmptyGroups(const StringX &path);
 
   // EmptyGroups
   std::vector<StringX> m_vEmptyGroups;
