@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -21,6 +21,7 @@
 #include "os/debug.h"
 #include "os/pws_tchar.h"
 #include "os/file.h"
+#include "os/env.h"
 #include "os/dir.h"
 #include "os/registry.h"
 #include "os/logit.h"
@@ -28,6 +29,7 @@
 #include <fstream>
 #include <algorithm>
 #include <iomanip>
+#include <climits>
 
 #ifdef _WIN32
 #include <LMCons.h> // for UNLEN
@@ -124,13 +126,18 @@ const PWSprefs::boolPref PWSprefs::m_bool_prefs[NumBoolPrefs] = {
   {_T("LockDBOnIdleTimeout"), true, ptDatabase},            // database
   {_T("HighlightChanges"), true, ptApplication},            // application
   {_T("HideSystemTray"), false, ptApplication},             // application
-  {_T("UsePrimarySelectionForClipboard"), false, ptApplication}, //application
+  {_T("UsePrimarySelectionForClipboard"), false, ptApplication}, // application
   {_T("CopyPasswordWhenBrowseToURL"), false, ptDatabase},   // database
-  {_T("UseAltAutoType"), false, ptApplication},             //application
-  {_T("IgnoreHelpLoadError"), false, ptApplication},        //application
-  {_T("VKPlaySound"), false, ptApplication},                //application
-  {_T("ListSortAscending"), true, ptApplication},           //application
-  {_T("EnableWindowTransparency"), false, ptApplication },  //application
+  {_T("UseAltAutoType"), false, ptApplication},             // application
+  {_T("IgnoreHelpLoadError"), false, ptApplication},        // application
+  {_T("VKPlaySound"), false, ptApplication},                // application
+  {_T("ListSortAscending"), true, ptApplication},           // application
+  {_T("EnableWindowTransparency"), false, ptApplication},   // application
+  {_T("ShowMenuSeparator"), true, ptApplication},           // application
+  {_T("AutoAdjColWidth"), false, ptApplication},            // application
+  {_T("ToolbarShowText"), true, ptApplication},             // application
+  {_T("DragAndDropShowRoot"), false, ptApplication},        // application
+  {_T("ShowAliasSelection"), false, ptApplication},         // application
 };
 
 // Default value = -1 means set at runtime
@@ -178,6 +185,7 @@ const PWSprefs::intPref PWSprefs::m_int_prefs[NumIntPrefs] = {
   {_T("VKFontPtSz"), 0, ptApplication, 0, -1},                      // application
   {_T("WindowTransparency"), 0, ptApplication, 0, 50},              // application
   {_T("DefaultExpiryDays"), 90, ptApplication, 1, 3650},            // application
+  {_T("DNDMaximumMemorySize"), 14000, ptApplication, -1, INT_MAX},   // application
 };
 
 const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
@@ -209,6 +217,8 @@ const PWSprefs::stringPref PWSprefs::m_string_prefs[NumStringPrefs] = {
   {_T("AddEditFont"), _T(""), ptApplication },                      // application
   {_T("AddEditSampleText"), _T("AaBbYyZz 0O1IlL"), ptApplication},  // application
   {_T("AltNotesEditorCmdLineParms"), _T(""), ptApplication},        // application
+  {_T("TreeSort"), _T("group"), ptApplication},                     // application
+
 };
 
 PWSprefs *PWSprefs::GetInstance()
@@ -288,7 +298,7 @@ unsigned int PWSprefs::GetPrefDefVal(IntPrefs pref_enum) const
   return m_int_prefs[pref_enum].defVal;
 }
 
-const TCHAR* const PWSprefs::GetPrefDefVal(StringPrefs pref_enum) const
+const TCHAR*  PWSprefs::GetPrefDefVal(StringPrefs pref_enum) const
 {
   return m_string_prefs[pref_enum].defVal;
 }
@@ -340,7 +350,7 @@ StringX PWSprefs::GetAllStringPrefs(const bool bUseCopy)
     LastView, TreeFont, BackupPrefixValue, ListColumns,
     ColumnWidths, MainToolBarButtons, PasswordFont,
     TreeListSampleText, PswdSampleText,
-    LastUsedKeyboard, VKeyboardFontName, VKSampleText, LanguageFile
+    LastUsedKeyboard, VKeyboardFontName, VKSampleText, LanguageFile, TreeSort
   };
 
   TCHAR delim;
@@ -1063,10 +1073,10 @@ void PWSprefs::FindConfigFile()
    */
 
   const stringT sExecDir = PWSdirs::GetExeDir();
+  const stringT sCnfgDir = PWSdirs::GetConfigDir();
 
   // Set path & name of config file
   if (!m_userSetCfgFile) { // common case
-    const stringT sCnfgDir = PWSdirs::GetConfigDir();
     m_configfilename = sExecDir + cfgFileName;
     if (pws_os::FileExists(m_configfilename)) {
       // old (exe dir) exists, is host/user there?
@@ -1076,13 +1086,9 @@ void PWSprefs::FindConfigFile()
     // not in exe dir or host/user not there
     m_configfilename = sCnfgDir + cfgFileName;
   } else { // User specified config file via SetConfigFile()
-    // As per pre-use of Local AppData directory,
-    // If file name's relative, it's expected to be in the
-    // same directory as the executable
-    stringT sDrive, sDir, sFile, sExt;
-    pws_os::splitpath(m_configfilename, sDrive, sDir, sFile, sExt);
-    if (sDrive.empty() || sDir.empty())
-      m_configfilename = sExecDir + sFile + sExt;
+    // In Windows, file path is made absolute in ThisMfcApp::GetConfigFromCommandLine(), nothing more to do
+    if (!pws_os::IsWindows() && m_configfilename[0] != L'/') // make absolute for non-Windows, if needed
+        m_configfilename = sCnfgDir + m_configfilename;
   }
 }
 
@@ -1476,6 +1482,9 @@ bool PWSprefs::LoadProfileFromFile()
   m_PSSrect.left = m_pXML_Config->Get(m_csHKCU_POS, _T("PSS_left"), -1);
   m_PSSrect.right = m_pXML_Config->Get(m_csHKCU_POS, _T("PSS_right"), -1);
 
+  // The XML tag 'Layout'
+  m_PrefLayout = m_pXML_Config->Get(m_csHKCU_PREF, _T("Layout"), L"");
+
   // Load most recently used file list
   for (i = m_intValues[MaxMRUItems]; i > 0; i--) {
     Format(csSubkey, L"Safe%02d", i);
@@ -1577,18 +1586,11 @@ void PWSprefs::SaveApplicationPreferences()
         break;
       case CF_FILE_RW:
       case CF_FILE_RW_NEW:
-      {
-        stringT obuff;
-        Format(obuff, L"%d", m_rect.top);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"top", obuff) == 0);
-        Format(obuff, L"%d", m_rect.bottom);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"bottom", obuff) == 0);
-        Format(obuff, L"%d", m_rect.left);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"left", obuff) == 0);
-        Format(obuff, L"%d", m_rect.right);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"right", obuff) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"top", static_cast<int>(m_rect.top)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"bottom", static_cast<int>(m_rect.bottom)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"left", static_cast<int>(m_rect.left)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"right", static_cast<int>(m_rect.right)) == 0);
         break;
-      }
       case CF_FILE_RO:
       case CF_NONE:
       default:
@@ -1611,25 +1613,23 @@ void PWSprefs::SaveApplicationPreferences()
         break;
       case CF_FILE_RW:
       case CF_FILE_RW_NEW:
-      {
-        stringT obuff;
-        Format(obuff, L"%d", m_PSSrect.top);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_top", obuff) == 0);
-        Format(obuff, L"%d", m_PSSrect.bottom);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_bottom", obuff) == 0);
-        Format(obuff, L"%d", m_PSSrect.left);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_left", obuff) == 0);
-        Format(obuff, L"%d", m_PSSrect.right);
-        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_right", obuff) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_top", static_cast<int>(m_PSSrect.top)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_bottom", static_cast<int>(m_PSSrect.bottom)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_left", static_cast<int>(m_PSSrect.left)) == 0);
+        VERIFY(m_pXML_Config->Set(m_csHKCU_POS, L"PSS_right", static_cast<int>(m_PSSrect.right)) == 0);
         break;
-      }
       case CF_FILE_RO:
       case CF_NONE:
       default:
         break;
     }
-    m_rect.changed = false;
-  } // m_rect.changed
+    m_PSSrect.changed = false;
+  } // m_PSSrect.changed
+
+  if (m_ConfigOption == CF_FILE_RW ||
+      m_ConfigOption == CF_FILE_RW_NEW) {
+    VERIFY(m_pXML_Config->Set(m_csHKCU_PREF, L"Layout", m_PrefLayout, pugi::xml_node_type::node_cdata) == 0);
+  }
 
   if (m_ConfigOption == CF_FILE_RW ||
       m_ConfigOption == CF_FILE_RW_NEW) {
@@ -1654,11 +1654,8 @@ void PWSprefs::SaveApplicationPreferences()
       m_ConfigOption == CF_FILE_RW_NEW) {
     if (m_pXML_Config->XML_Store(m_csHKCU_PREF)) // can't be new after succ. store
       m_ConfigOption = CF_FILE_RW;
-    else
-    if (!m_pXML_Config->getReason().empty() &&
-        m_pReporter != nullptr)
+    else if (!m_pXML_Config->getReason().empty() && m_pReporter != nullptr)
       (*m_pReporter)(m_pXML_Config->getReason()); // show what went wrong
-
     m_pXML_Config->Unlock();
     delete m_pXML_Config;
     m_pXML_Config = nullptr;
