@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+* Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
 * All rights reserved. Use of the code is allowed under the
 * Artistic License 2.0 terms, as specified in the LICENSE file
 * distributed with this code, or available from
@@ -128,6 +128,11 @@ public:
     VKPlaySound, // Windows only
     ListSortAscending,
     EnableWindowTransparency,
+    ShowMenuSeparator,
+    AutoAdjColWidth,
+    ToolbarShowText,
+    DragAndDropShowRoot,
+    ShowAliasSelection,
     NumBoolPrefs};
 
   enum IntPrefs {Column1Width, Column2Width, Column3Width, Column4Width,
@@ -140,7 +145,7 @@ public:
     DlgOrientation, TimedTaskChainDelay,
     AutotypeSelectAllKeyCode, AutotypeSelectAllModMask, //X only
     TreeFontPtSz, PasswordFontPtSz, NotesFontPtSz, AddEditFontPtSz, VKFontPtSz,
-    WindowTransparency, DefaultExpiryDays,
+    WindowTransparency, DefaultExpiryDays, DNDMaxMemSize,
     NumIntPrefs};
 
   enum StringPrefs {CurrentBackup, CurrentFile, LastView, DefaultUsername,
@@ -149,7 +154,7 @@ public:
     MainToolBarButtons, PasswordFont, TreeListSampleText, PswdSampleText,
     LastUsedKeyboard, VKeyboardFontName, VKSampleText, AltNotesEditor,
     LanguageFile, DefaultSymbols, NotesFont, NotesSampleText, AutotypeTaskDelays,
-    AddEditFont, AddEditSampleText, AltNotesEditorCmdLineParms,
+    AddEditFont, AddEditSampleText, AltNotesEditorCmdLineParms, TreeSort,
     NumStringPrefs};
 
   // for DoubleClickAction and ShiftDoubleClickAction
@@ -202,7 +207,7 @@ public:
 
   bool GetPrefDefVal(BoolPrefs pref_enum) const;
   unsigned int GetPrefDefVal(IntPrefs pref_enum) const;
-  const TCHAR* const GetPrefDefVal(StringPrefs pref_enum) const;
+    const TCHAR* GetPrefDefVal(StringPrefs pref_enum) const;
 
   int GetPrefMinVal(IntPrefs pref_enum) const;
   int GetPrefMaxVal(IntPrefs pref_enum) const;
@@ -241,7 +246,11 @@ public:
   // for display in status bar (debug)
   int GetConfigIndicator() const;
 
-  // Get & set vector of user shortcuts (only in XML cnfig file)
+  // Get & set layout data (only in XML config file)
+  stringT GetPrefLayout() const { return m_PrefLayout; };
+  void SetPrefLayout(const stringT& preferences) { m_PrefLayout = preferences; };
+
+  // Get & set vector of user shortcuts (only in XML config file)
   std::vector<st_prefShortcut> GetPrefShortcuts() const {return m_vShortcuts;}
   void SetPrefShortcuts(const std::vector<st_prefShortcut> &vShortcuts);
 
@@ -315,6 +324,7 @@ private:
   StringX m_stringCopyValues[NumStringPrefs];
 
   stringT *m_MRUitems;
+  stringT m_PrefLayout;
   std::vector<st_prefShortcut> m_vShortcuts;
 
   // Preferences we don't know in this version of PWS
