@@ -104,7 +104,7 @@
 #define IDSC_IMPORTLINESKIPPED          3259
 #define IDSC_IMPORTKNOWNHDRS            3260
 #define IDSC_IMPORTEMPTYLINESKIPPED     3261
-#define IDSC_PWHERRORLINE               3262
+/* #define IDSC_PWHERRORLINE               3262 */ // Not defined and not used
 #define IDSC_PWHISTORYSKIPPED           3263
 #define IDSC_IMPORTCONFLICTS1           3264
 #define IDSC_IMPORTCONFLICTS2           3265
@@ -265,7 +265,6 @@
 #define IDSC_VALIDATE_EMPTYTITLE        3437
 #define IDSC_VALIDATE_EMPTYPSWD         3438
 #define IDSC_VALIDATE_DUPUUID           3439
-#define IDSC_RPTVALIDATE                3440
 #define IDSC_VALIDATE_DEPS              3441
 #define IDSC_VALIDATE_ERRORS            3442
 #define IDSC_VALIDATE_WARNINGS          3443
@@ -332,5 +331,26 @@
 #define IDSC_PHEXADECIMAL               4113
 #define IDSC_DEFAULT_POLICY             4114
 #define IDSC_ATLEASTIGNORED             4115
-
 #define IDSC_INVALIDLOCKER              4116
+
+#define IDSC_COMPARESTATISTICS          5356
+#define IDSC_COMPARESTATS               5359
+#define IDSC_DRAGNUMBER                 5411
+#define IDSC_RPTCOMPARE                 5532
+#define IDSC_RPTFIND                    5533
+#define IDSC_RPTIMPORTTEXT              5534
+#define IDSC_RPTIMPORTXML               5535
+#define IDSC_RPTMERGE                   5536
+#define IDSC_RPTVALIDATE                5537
+#define IDSC_SEARCHRESULTS1             5538
+#define IDSC_SEARCHRESULTS2             5539
+#define IDSC_RPTSYNCH                   5749
+#define IDSC_RPTEXPORTTEXT              5805
+#define IDSC_RPTEXPORTXML               5806
+#define IDSC_RPTIMPORTKPV1TXT           5868
+#define IDSC_RPTIMPORTKPV1CSV           5869
+#define IDSC_DRAGPOLICY                 5907
+#define IDSC_RPTEXPORTDB                5965
+
+#define IDSC_RELATIVE_SMALL             5968
+#define IDSC_AND_SMALL                  5969
