@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -43,7 +43,8 @@ DbSelectionPanel::DbSelectionPanel(wxWindow* parent,
                                     bool autoValidate,
                                     PWScore* core,
                                     unsigned rowsep,
-                                    int buttonConfirmationId) : wxPanel(parent), m_pollingTimer(nullptr),
+                                    int buttonConfirmationId,
+                                    const wxString filename) : wxPanel(parent), m_pollingTimer(nullptr),
                                                                 m_filepicker(nullptr),
                                                                 m_sc(nullptr),
                                                                 m_bAutoValidate(autoValidate),
@@ -62,8 +63,9 @@ DbSelectionPanel::DbSelectionPanel(wxWindow* parent,
 
   panelSizer->Add(new wxStaticText(this, wxID_ANY, filePrompt), borderFlags);
   panelSizer->AddSpacer(RowSeparation);
+  m_filepath = filename;
   OpenFilePickerValidator validator(m_filepath);
-  m_filepicker = new wxFilePickerCtrl(this, wxID_ANY, wxEmptyString, 
+  m_filepicker = new wxFilePickerCtrl(this, wxID_ANY, wxEmptyString,
                                           filePickerCtrlTitle,
                                           _("Password Safe Databases (*.psafe4; *.psafe3; *.dat)|*.psafe4;*.psafe3;*.dat|Password Safe Backups (*.bak)|*.bak|Password Safe Intermediate Backups (*.ibak)|*.ibak|All files (*.*; *)|*.*;*"), 
                                           wxDefaultPosition, wxDefaultSize, 
@@ -117,7 +119,7 @@ DbSelectionPanel::DbSelectionPanel(wxWindow* parent,
 #endif
   
   //The parent window must call our TransferDataToWindow and TransferDataFromWindow
-  m_parent->SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
+  m_parent->SetExtraStyle(m_parent->GetExtraStyle() | wxWS_EX_VALIDATE_RECURSIVELY);
 }
 
 DbSelectionPanel::~DbSelectionPanel()
