@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -27,10 +27,12 @@
 #include <wx/tokenzr.h>
 #include <wx/utils.h> // for wxLaunchDefaultBrowser
 
+#include "core/core.h"
 #include "core/PWSdirs.h"
 #include "core/XML/XMLDefs.h"
 #include "os/sleep.h"
 #include "os/file.h"
+#include "os/env.h"
 
 #include "CompareDlg.h"
 #include "ExportTextWarningDlg.h"
@@ -48,7 +50,7 @@
 #include "TreeCtrl.h"
 #include "wxUtilities.h"
 
-static void DisplayFileWriteError(int rc, const StringX &fname)
+void PasswordSafeFrame::DisplayFileWriteError(int rc, const StringX &fname)
 {
   ASSERT(rc != PWScore::SUCCESS);
 
@@ -68,7 +70,8 @@ static void DisplayFileWriteError(int rc, const StringX &fname)
     cs_temp += _("Unknown error");
     break;
   }
-  wxMessageDialog(nullptr, cs_temp, cs_title, wxOK | wxICON_ERROR);
+  wxMessageDialog dlg(nullptr, cs_temp, cs_title, wxOK | wxICON_ERROR);
+  dlg.ShowModal();
 }
 
 /*!
@@ -77,7 +80,7 @@ static void DisplayFileWriteError(int rc, const StringX &fname)
 
 void PasswordSafeFrame::OnNewClick(wxCommandEvent& WXUNUSED(evt))
 {
-  New();
+  CallAfter([&](){PasswordSafeFrame::New();});
 }
 
 int PasswordSafeFrame::New()
@@ -147,7 +150,8 @@ int PasswordSafeFrame::NewFile(StringX &fname)
   wxString dir = towxstring(PWSdirs::GetSafeDir());
   int rc;
 
-  while (1) {
+  while (1) { // Lock cannot be fetched or Cancel pressed
+    
     wxFileDialog fd(static_cast<wxWindow *>(this), cs_text, dir, v3FileName,
                     _("psafe3 files (*.psafe3)|*.psafe3|All files(*.*; *)|*.*;*"),
                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT | wxFD_CHANGE_DIR);
@@ -160,43 +164,69 @@ int PasswordSafeFrame::NewFile(StringX &fname)
         wxfn.SetExt(DEFAULT_SUFFIX);
         fname = wxfn.GetFullPath().c_str();
       }
-      break;
     } else
       return PWScore::USER_CANCEL;
-  }
 
-  SafeCombinationSetupDlg dbox_pksetup(this);
-  rc = dbox_pksetup.ShowModal();
+    DestroyWrapper<SafeCombinationSetupDlg> dbox_pksetupWrapper(this);
+    SafeCombinationSetupDlg* dbox_pksetup = dbox_pksetupWrapper.Get();
+    rc = dbox_pksetup->ShowModal();
 
-  if (rc == wxID_CANCEL)
-    return PWScore::USER_CANCEL;  //User cancelled password entry
+    if (rc == wxID_CANCEL)
+      return PWScore::USER_CANCEL;  //User cancelled password entry
 
-  // Reset core and clear ALL associated data
-  m_core.ReInit(true);
+    // First lock the new file
+    std::wstring locker(L""); // null init is important here
+    if(!m_core.LockFile(fname.c_str(), locker)) {
+      stringT plkUser(_T(""));
+      stringT plkHost(_T(""));
+      int plkPid = -1;
+      bool fileLocked = false;
+      if(PWSUtil::GetLockerData(locker, plkUser, plkHost, plkPid) &&
+         (plkUser == pws_os::getusername()) && (plkHost == pws_os::gethostname())) {
+        wxMessageDialog dialog(this, _("Lock is done by yourself"), _("Remove Lock?"), wxYES_NO | wxICON_EXCLAMATION);
+        if(dialog.ShowModal() == wxID_YES) {
+          HANDLE handle = 0;
+          pws_os::UnlockFile(fname.c_str(), handle);
+          if(m_core.LockFile(fname.c_str(), locker))
+            fileLocked = true;
+        }
+      }
+      if(!fileLocked) {
+        wxString errmess;
+        errmess = _("Could not lock file.\n");
+        if (PWSUtil::HasValidLockerData(locker)) {
+          errmess += _("Locked by ");
+        }
+        errmess += locker.c_str();
+        wxMessageBox(wxString()<< fname.c_str() << wxT("\n\n") << errmess,
+                     _("Error"), wxOK | wxICON_ERROR, this);
+        continue;
+      }
+    }
+    // Reset core and clear ALL associated data
+    m_core.ReInit(true);
 
-  // clear the application data before creating new file
-  ClearAppData();
+    // clear the application data before creating new file
+    ClearAppData();
 
-  PWSprefs::GetInstance()->SetDatabasePrefsToDefaults();
-  const StringX &oldfilename = m_core.GetCurFile();
-  // The only way we're the locker is if it's locked & we're !readonly
-  if (!oldfilename.empty() &&
-      !m_core.IsReadOnly() &&
-      m_core.IsLockedFile(oldfilename.c_str()))
-    m_core.UnlockFile(oldfilename.c_str());
+    PWSprefs::GetInstance()->SetDatabasePrefsToDefaults();
+    const StringX &oldfilename = m_core.GetCurFile();
+    // The only way we're the locker is if it's locked & we're !readonly
+    if (!oldfilename.empty() &&
+        !m_core.IsReadOnly() &&
+        m_core.IsLockedFile(oldfilename.c_str())) {
+      m_core.UnlockFile(oldfilename.c_str());
+    }
 
-  m_core.SetCurFile(fname);
+    m_core.SetCurFile(fname);
 
-  // Now lock the new file
-  std::wstring locker(L""); // null init is important here
-  m_core.LockFile(fname.c_str(), locker);
-
-  m_core.SetReadOnly(false); // new file can't be read-only...
-  m_core.NewFile(tostringx(dbox_pksetup.GetPassword()));
+    m_core.SetReadOnly(false); // new file can't be read-only...
+    m_core.NewFile(tostringx(dbox_pksetup->GetPassword()));
 #ifdef notyet
-  startLockCheckTimer();
+    startLockCheckTimer();
 #endif
-  return PWScore::SUCCESS;
+    return PWScore::SUCCESS;
+  }
 }
 
 /*!
@@ -219,32 +249,7 @@ void PasswordSafeFrame::OnOpenClick(wxCommandEvent& WXUNUSED(evt))
 
 void PasswordSafeFrame::OnCloseClick(wxCommandEvent& WXUNUSED(evt))
 {
-  PWSprefs *prefs = PWSprefs::GetInstance();
-
-  // Save Application related preferences
-  prefs->SaveApplicationPreferences();
-  if( m_core.IsDbOpen() ) {
-    int rc = SaveIfChanged();
-    if (rc != PWScore::SUCCESS)
-      return;
-
-    m_core.SafeUnlockCurFile();
-    m_core.SetCurFile(wxEmptyString);
-
-    // Reset core and clear ALL associated data
-    m_core.ReInit();
-
-    // clear the application data before ending
-    ClearAppData();
-
-    SetTitle(wxEmptyString);
-    m_sysTray->SetTrayStatus(SystemTray::TrayStatus::CLOSED);
-    wxCommandEvent dummyEv;
-    m_search->OnSearchClose(dummyEv); // fix github issue 375
-    m_core.SetReadOnly(false);
-    UpdateStatusBar();
-    UpdateMenuBar();
-  }
+  CloseDB(nullptr);
 }
 
 void PasswordSafeFrame::OnLockSafe(wxCommandEvent&)
@@ -254,7 +259,7 @@ void PasswordSafeFrame::OnLockSafe(wxCommandEvent&)
 
 void PasswordSafeFrame::OnUnlockSafe(wxCommandEvent&)
 {
-  UnlockSafe(true, true);
+  CallAfter(&PasswordSafeFrame::UnlockSafe, true, true);
 }
 
 /*!
@@ -585,6 +590,7 @@ int PasswordSafeFrame::SaveIfChanged()
 
 struct ExportFullText
 {
+  static wxString EngGetTitle() {return CReport::ReportNames.find(IDSC_RPTEXPORTTEXT)->second;}
   static wxString GetTitle() {return _("Export Text");}
   static void MakeOrderedItemList(PasswordSafeFrame* frame, OrderedItemList& olist) {
     frame->FlattenTree(olist);
@@ -627,6 +633,7 @@ struct ExportFullText
 };
 
 struct ExportFullXml {
+  static wxString EngGetTitle() {return CReport::ReportNames.find(IDSC_RPTEXPORTXML)->second;}
   static wxString GetTitle() {return _("Export XML");}
   static void MakeOrderedItemList(PasswordSafeFrame* frame, OrderedItemList& olist) {
     frame->FlattenTree(olist);
@@ -726,13 +733,13 @@ void PasswordSafeFrame::OnExportVx(wxCommandEvent& evt)
 void PasswordSafeFrame::OnExportPlainText(wxCommandEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
-  DoExportText<ExportFullText>();
+  CallAfter(&PasswordSafeFrame::DoExportText<ExportFullText>);
 }
 
 void PasswordSafeFrame::OnExportXml(wxCommandEvent& evt)
 {
   UNREFERENCED_PARAMETER(evt);
-  DoExportText<ExportFullXml>();
+  CallAfter(&PasswordSafeFrame::DoExportText<ExportFullXml>);
 }
 
 IMPLEMENT_CLASS_TEMPLATE( AdvancedSelectionDlg, wxDialog, ExportFullXml )
@@ -753,18 +760,20 @@ void PasswordSafeFrame::DoExportText()
     return;
   }
 
-  ExportTextWarningDlg<ExportType> et(this);
-  if (et.ShowModal() != wxID_OK)
+  DestroyWrapper<ExportTextWarningDlg<ExportType>> etWrapper(this);
+  auto *et = etWrapper.Get(); 
+  if (et->ShowModal() != wxID_OK) {
     return;
+  }
 
   StringX newfile;
-  StringX pw(et.passKey);
+  StringX pw(et->passKey);
   if (m_core.CheckPasskey(sx_temp, pw) == PWScore::SUCCESS) {
-    const CItemData::FieldBits bsExport = et.selCriteria->GetSelectedFields();
-    const std::wstring subgroup_name = tostdstring(et.selCriteria->SubgroupSearchText());
-    const int subgroup_object = et.selCriteria->SubgroupObject();
-    const int subgroup_function = et.selCriteria->SubgroupFunctionWithCase();
-    wchar_t delimiter = et.delimiter.IsEmpty()? wxT('\xbb') : et.delimiter[0];
+    const CItemData::FieldBits bsExport = et->selCriteria->GetSelectedFields();
+    const std::wstring subgroup_name = tostdstring(et->selCriteria->SubgroupSearchText());
+    const int subgroup_object = et->selCriteria->SubgroupObject();
+    const int subgroup_function = et->selCriteria->SubgroupFunctionWithCase();
+    wchar_t delimiter = et->delimiter.IsEmpty()? wxT('\xbb') : et->delimiter[0];
 
     // Note: MakeOrderedItemList gets its members by walking the
     // tree therefore, if a filter is active, it will ONLY export
@@ -794,8 +803,8 @@ void PasswordSafeFrame::DoExportText()
           newfile = fd.GetPath().c_str();
           CReport rpt;
 
-          rpt.StartReport(ExportType::GetTitle().c_str(), sx_temp.c_str());
-          rpt.WriteLine(tostdstring(wxString(_("Exporting database: ")) << towxstring(sx_temp) << wxT(" to ") << newfile<< wxT("\r\n")));
+          rpt.StartReport(IDSC_RPTEXPORTTEXT, sx_temp.c_str());
+          rpt.WriteLine(tostdstring(wxString(_("Exporting database: ")) << towxstring(sx_temp) << _(" to ") << newfile<< wxT("\r\n")));
 
           int rc = ExportType::Write(m_core, newfile, bsExport, subgroup_name, subgroup_object,
                                       subgroup_function, delimiter, numExported, &orderedItemList, &rpt);
@@ -839,7 +848,11 @@ void PasswordSafeFrame::DoExportText()
 
 void PasswordSafeFrame::OnImportText(wxCommandEvent& evt)
 {
-  UNREFERENCED_PARAMETER(evt);
+  CallAfter(&PasswordSafeFrame::DoImportText, evt.GetString());
+}
+
+void PasswordSafeFrame::DoImportText(wxString filename)
+{
   if (m_core.IsReadOnly()) {// disable in read-only mode
     wxMessageBox(_("The current database was opened in read-only mode.  You cannot import into it."),
                   _("Import text"), wxOK | wxICON_EXCLAMATION, this);
@@ -857,22 +870,23 @@ void PasswordSafeFrame::OnImportText(wxCommandEvent& evt)
     return;
   }
 
-  ImportTextDlg dlg(this);
-  if (dlg.ShowModal() != wxID_OK)
+  DestroyWrapper<ImportTextDlg> dlgWrapper(this, filename);
+  ImportTextDlg *dlg = dlgWrapper.Get();
+  if (dlg->ShowModal() != wxID_OK)
     return;
 
-  StringX ImportedPrefix(dlg.groupName.c_str());
-  TCHAR fieldSeparator = dlg.FieldSeparator();
+  StringX ImportedPrefix(dlg->groupName.c_str());
+  TCHAR fieldSeparator = dlg->FieldSeparator();
 
   std::wstring strError;
-  wxString TxtFileName = dlg.filepath;
+  wxString TxtFileName = dlg->filepath;
   int numImported(0), numSkipped(0), numPWHErrors(0), numRenamed(0), numNoPolicyNames(0);
-  wchar_t delimiter = dlg.strDelimiterLine[0];
-  bool bImportPSWDsOnly = dlg.importPasswordsOnly;
+  wchar_t delimiter = dlg->strDelimiterLine[0];
+  bool bImportPSWDsOnly = dlg->importPasswordsOnly;
 
   /* Create report as we go */
   CReport rpt;
-  rpt.StartReport(_("Import_Text").c_str(), m_core.GetCurFile().c_str());
+  rpt.StartReport(IDSC_RPTEXPORTTEXT, m_core.GetCurFile().c_str());
   wxString header;
   header.Printf(_("%ls file being imported: %ls"), _("Text"), TxtFileName.c_str());
   rpt.WriteLine(tostdstring(header));
@@ -957,7 +971,11 @@ void PasswordSafeFrame::OnImportText(wxCommandEvent& evt)
 
 void PasswordSafeFrame::OnImportXML(wxCommandEvent& evt)
 {
-  UNREFERENCED_PARAMETER(evt);
+  CallAfter(&PasswordSafeFrame::DoImportXML, evt.GetString());
+}
+
+void PasswordSafeFrame::DoImportXML(wxString filename)
+{
   if (m_core.IsReadOnly()) // disable in read-only mode
     return;
 
@@ -982,29 +1000,31 @@ void PasswordSafeFrame::OnImportXML(wxCommandEvent& evt)
   }
 #endif
 
-  ImportXmlDlg dlg(this);
-  if (dlg.ShowModal() != wxID_OK)
+  DestroyWrapper<ImportXmlDlg> dlgWrapper(this, filename);
+  ImportXmlDlg *dlg = dlgWrapper.Get();
+  if (dlg->ShowModal() != wxID_OK) {
     return;
+  }
 
-  std::wstring ImportedPrefix(tostdstring(dlg.groupName));
+  std::wstring ImportedPrefix(tostdstring(dlg->groupName));
   std::wstring strXMLErrors, strSkippedList, strPWHErrorList, strRenameList;
-  wxString XMLFilename = dlg.filepath;
+  wxString XMLFilename = dlg->filepath;
   int numValidated, numImported, numSkipped, numRenamed, numPWHErrors;
   int numRenamedPolicies, numNoPolicy;
   int numShortcutsRemoved, numEmptyGroupsImported;
-  bool bImportPSWDsOnly = dlg.importPasswordsOnly;
+  bool bImportPSWDsOnly = dlg->importPasswordsOnly;
 
   wxBeginBusyCursor();  // This may take a while!
 
   /* Create report as we go */
   CReport rpt;
-  rpt.StartReport(_("Import_XML").c_str(), m_core.GetCurFile().c_str());
+  rpt.StartReport(IDSC_RPTIMPORTXML, m_core.GetCurFile().c_str());
   rpt.WriteLine(tostdstring(wxString::Format(_("%ls file being imported: %ls"), _("XML"), XMLFilename.c_str())));
   rpt.WriteLine();
   std::vector<StringX> vgroups;
   Command *pcmd = nullptr;
 
-  int rc = m_core.ImportXMLFile(ImportedPrefix, std::wstring(XMLFilename),
+  int rc = m_core.ImportXMLFile(ImportedPrefix, tostdstring(XMLFilename),
                             tostdstring(XSDFilename.GetFullPath()), bImportPSWDsOnly,
                             strXMLErrors, strSkippedList, strPWHErrorList, strRenameList,
                             numValidated, numImported, numSkipped, numPWHErrors, numRenamed,
@@ -1019,15 +1039,13 @@ void PasswordSafeFrame::OnImportXML(wxCommandEvent& evt)
   std::wstring csErrors(wxEmptyString);
   switch (rc) {
     case PWScore::XML_FAILED_VALIDATION:
-      rpt.WriteLine(strXMLErrors.c_str());
       cs_temp = wxString::Format(_("File: %ls failed validation against XML Schema:\n\n%ls"),
-                                        dlg.filepath.c_str(), wxEmptyString);
+                                        dlg->filepath.c_str(), strXMLErrors.c_str());
       delete pcmd;
       break;
     case PWScore::XML_FAILED_IMPORT:
-      rpt.WriteLine(strXMLErrors.c_str());
       cs_temp = wxString::Format(_("File: %ls passed Validation but had the following errors during import:\n\n%ls"),
-                              dlg.filepath.c_str(), wxEmptyString);
+                              dlg->filepath.c_str(), strXMLErrors.c_str());
       delete pcmd;
       break;
     case PWScore::SUCCESS:
@@ -1069,7 +1087,7 @@ void PasswordSafeFrame::OnImportXML(wxCommandEvent& evt)
         }
 
         cs_temp.Printf(_("File: %ls was imported (entries validated %d / imported %d%ls%ls%ls). See report for details."),
-                       dlg.filepath.c_str(), numValidated, numImported,
+                       dlg->filepath.c_str(), numValidated, numImported,
                        cs_skipped.c_str(), cs_renamed.c_str(), cs_PWHErrors.c_str());
         // TODO -Tell user if any empty groups imported
       } else {
@@ -1104,27 +1122,38 @@ void PasswordSafeFrame::OnImportXML(wxCommandEvent& evt)
 
 void PasswordSafeFrame::OnImportKeePass(wxCommandEvent& evt)
 {
-  UNREFERENCED_PARAMETER(evt);
+  CallAfter(&PasswordSafeFrame::DoImportKeePass, evt.GetString());
+}
+
+void PasswordSafeFrame::DoImportKeePass(wxString filename)
+{
   if (m_core.IsReadOnly()) // disable in read-only mode
     return;
 
-  wxFileDialog fd(this, _("Please Choose a KeePass Text File to Import"),
-                  wxEmptyString, wxEmptyString,
+  wxString KPsFileName;
+  
+  if(filename.IsEmpty()) {
+    wxFileDialog fd(this, _("Please Choose a KeePass Text File to Import"),
+                  wxEmptyString, filename,
                   _("Text files (*.txt)|*.txt|CSV files (*.csv)|*.csv|All files (*.*; *)|*.*;*"),
                   (wxFD_OPEN | wxFD_FILE_MUST_EXIST | wxFD_PREVIEW));
 
-  if (fd.ShowModal() != wxID_OK )
-    return;
-
-  const wxString KPsFileName(fd.GetPath());
+    if (fd.ShowModal() != wxID_OK )
+      return;
+    
+    KPsFileName = fd.GetPath();
+  }
+  else {
+    KPsFileName = filename;
+  }
   CReport rpt;
 
   enum { KeePassCSV, KeePassTXT } ImportType = wxFileName(KPsFileName).GetExt() == wxT("csv")? KeePassCSV: KeePassTXT;
 
   if (ImportType == KeePassCSV)
-    rpt.StartReport(_("Import_KeePassV1_CSV").c_str(), m_core.GetCurFile().c_str());
+    rpt.StartReport(IDSC_RPTIMPORTKPV1CSV, m_core.GetCurFile().c_str());
   else
-    rpt.StartReport(_("Import_KeePassV1_TXT").c_str(), m_core.GetCurFile().c_str());
+    rpt.StartReport(IDSC_RPTIMPORTKPV1TXT, m_core.GetCurFile().c_str());
 
   rpt.WriteLine(wxString::Format(_("Text file being imported: %ls"), KPsFileName.c_str()));
   rpt.WriteLine();
@@ -1157,9 +1186,12 @@ void PasswordSafeFrame::OnImportKeePass(wxCommandEvent& evt)
         LoadAString(s, uiReasonCode);
         msg = towxstring(s);
       }
-      else
+      else {
         msg = wxString::Format(_("%ls\n\nInvalid format"), KPsFileName.GetData());
-      wxMessageBox(msg, _("Import failed"), wxOK | wxICON_ERROR, this);
+      }
+      msg << wxT("\n\n") << _("Do you wish to see a detailed report?");
+      if (wxMessageBox(msg, _("Import failed"), wxYES_NO | wxICON_ERROR, this) == wxYES)
+        ViewReport(rpt);
       delete [] pcmd;
       break;
     }
@@ -1191,9 +1223,15 @@ void PasswordSafeFrame::OnImportKeePass(wxCommandEvent& evt)
 
 void PasswordSafeFrame::OnMergeAnotherSafe(wxCommandEvent& evt)
 {
-  UNREFERENCED_PARAMETER(evt);
-  MergeDlg dlg(this, &m_core);
-  if (dlg.ShowModal() == wxID_OK) {
+  CallAfter(&PasswordSafeFrame::DoMergeAnotherSafe, evt.GetString());
+}
+
+void PasswordSafeFrame::DoMergeAnotherSafe(wxString filename)
+{
+  DestroyWrapper<MergeDlg> dlgWrapper(this, &m_core, filename);
+  MergeDlg* dlg = dlgWrapper.Get();
+
+  if (dlg->ShowModal() == wxID_OK) {
     PWScore othercore; // NOT PWSAuxCore, as we handle db prefs explicitly
     // Reading a new file changes the preferences as they are instance dependent
     // not core dependent
@@ -1205,14 +1243,14 @@ void PasswordSafeFrame::OnMergeAnotherSafe(wxCommandEvent& evt)
     // 'other' default Password Policy when needed in Compare, Merge & Sync
     prefs->SetupCopyPrefs();
 
-    int rc = ReadCore(othercore, dlg.GetOtherSafePath(),
-                      dlg.GetOtherSafeCombination(), true, this);
+    int rc = ReadCore(othercore, dlg->GetOtherSafePath(),
+                      dlg->GetOtherSafeCombination(), true, this);
 
     // Reset database preferences - first to defaults then add saved changes!
     prefs->Load(sxSavePrefString);
 
     if (rc == PWScore::SUCCESS) {
-        Merge(tostringx(dlg.GetOtherSafePath()), &othercore, dlg.GetSelectionCriteria());
+        Merge(tostringx(dlg->GetOtherSafePath()), &othercore, dlg->GetSelectionCriteria());
     }
   }
 }
@@ -1225,7 +1263,7 @@ void PasswordSafeFrame::Merge(const StringX &sx_Filename2, PWScore *pothercore,
   /* Create report as we go */
   CReport rpt;
 
-  rpt.StartReport(_("Merge").c_str(), m_core.GetCurFile().c_str());
+  rpt.StartReport(IDSC_RPTMERGE, m_core.GetCurFile().c_str());
   rpt.WriteLine(tostdstring(wxString(_("Merging database: ")) << towxstring(sx_Filename2) << wxT("\r\n")));
 
   stringT result = m_core.Merge(pothercore,
@@ -1248,55 +1286,72 @@ void PasswordSafeFrame::Merge(const StringX &sx_Filename2, PWScore *pothercore,
 
 void PasswordSafeFrame::OnCompare(wxCommandEvent& WXUNUSED(evt))
 {
-  CompareDlg dlg(this, &m_core);
-  dlg.ShowModal();
+  CallAfter(&PasswordSafeFrame::DoCompare);
 }
 
-void PasswordSafeFrame::OnSynchronize(wxCommandEvent& WXUNUSED(evt))
+void PasswordSafeFrame::DoCompare()
+{
+  ShowModalAndGetResult<CompareDlg>(this, &m_core);
+}
+
+void PasswordSafeFrame::OnSynchronize(wxCommandEvent& evt)
+{
+  CallAfter(&PasswordSafeFrame::DoSynchronize, evt.GetString());
+}
+
+void PasswordSafeFrame::DoSynchronize(wxString filename)
 {
   // disable in read-only mode or empty
   wxCHECK_RET(!m_core.IsReadOnly() && m_core.IsDbOpen() && m_core.GetNumEntries() != 0,
                 wxT("Synchronize menu enabled for empty or read-only database!"));
 
-  SyncWizard wiz(this, &m_core);
-  wiz.RunWizard(wiz.GetFirstPage());
-
-  if (wiz.GetNumUpdated() > 0)
-    UpdateStatusBar();
-
+  SyncWizard wiz(this, &m_core, filename);
+  if (wiz.RunWizard(wiz.GetFirstPage())) {
+    if (wiz.GetNumUpdated() > 0 && wiz.GetSyncCommands()) {
+      m_core.Execute(wiz.GetSyncCommands());
+      UpdateStatusBar();
+    }
 #ifdef NOT_YET
-  ChangeOkUpdate();
+    ChangeOkUpdate();
 #endif
 
-  RefreshViews();
+    RefreshViews();
 
-  if (wiz.ShowReport())
-    ViewReport(*wiz.GetReport());
+    if (wiz.ShowReport()) {
+      ViewReport(*wiz.GetReport());
+    }
+  }
 }
 
 void PasswordSafeFrame::OnPropertiesClick(wxCommandEvent& WXUNUSED(evt))
 {
-  PropertiesDlg propsDialog(this, m_core);
-  propsDialog.ShowModal();
+  CallAfter(&PasswordSafeFrame::DoPropertiesClick);
+}
 
-  if (propsDialog.HasDbNameChanged() || propsDialog.HasDbDescriptionChanged()) {
+void PasswordSafeFrame::DoPropertiesClick()
+{
+  DestroyWrapper<PropertiesDlg> dlgWrapper(this, m_core);
+  PropertiesDlg* propsDialog = dlgWrapper.Get();
+  propsDialog->ShowModal();
+
+  if (propsDialog->HasDbNameChanged() || propsDialog->HasDbDescriptionChanged()) {
 
     auto multiCommands = MultiCommands::Create(&m_core);
 
-    if (propsDialog.HasDbNameChanged()) {
+    if (propsDialog->HasDbNameChanged()) {
 
       multiCommands->Add(
         ChangeDBHeaderCommand::Create(
-          &m_core, propsDialog.GetNewDbName(), PWSfile::HDR_DBNAME
+          &m_core, propsDialog->GetNewDbName(), PWSfile::HDR_DBNAME
         )
       );
     }
 
-    if (propsDialog.HasDbDescriptionChanged()) {
+    if (propsDialog->HasDbDescriptionChanged()) {
 
       multiCommands->Add(
         ChangeDBHeaderCommand::Create(
-          &m_core, propsDialog.GetNewDbDescription(), PWSfile::HDR_DBDESC
+          &m_core, propsDialog->GetNewDbDescription(), PWSfile::HDR_DBDESC
         )
       );
     }
@@ -1316,8 +1371,10 @@ void PasswordSafeFrame::OnPropertiesClick(wxCommandEvent& WXUNUSED(evt))
 
 void PasswordSafeFrame::OnExitClick(wxCommandEvent& WXUNUSED(evt))
 {
-  m_exitFromMenu = true;
-
-  Close();
+  CloseAllWindows(&TimedTaskChain::CreateTaskChain([](){}), CloseFlags::CLOSE_NORMAL, [this](bool success) {
+    if (!success) {
+      // `this` should be valid here, because we haven't closed DB
+      wxMessageBox(_("Can't close database. There are unsaved changes in opened dialogs."), wxTheApp->GetAppName(), wxOK | wxICON_WARNING, this);
+    }
+  });
 }
-
