@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2020 Rony Shapiro <ronys@pwsafe.org>.
+ * Copyright (c) 2003-2022 Rony Shapiro <ronys@pwsafe.org>.
  * All rights reserved. Use of the code is allowed under the
  * Artistic License 2.0 terms, as specified in the LICENSE file
  * distributed with this code, or available from
@@ -66,22 +66,13 @@ END_EVENT_TABLE()
  * SafeCombinationChangeDlg constructors
  */
 
-SafeCombinationChangeDlg::SafeCombinationChangeDlg(wxWindow* parent, PWScore &core,
+SafeCombinationChangeDlg::SafeCombinationChangeDlg(wxWindow *parent, PWScore &core,
                                                wxWindowID id, const wxString& caption,
                                                const wxPoint& pos,
                                                const wxSize& size, long style)
 : m_core(core)
 {
-  Init();
-  Create(parent, id, caption, pos, size, style);
-}
-
-/*!
- * SafeCombinationChangeDlg creator
- */
-
-bool SafeCombinationChangeDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
-{
+  wxASSERT(!parent || parent->IsTopLevel());
 ////@begin SafeCombinationChangeDlg creation
   SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
   wxDialog::Create( parent, id, caption, pos, size, style );
@@ -103,7 +94,13 @@ bool SafeCombinationChangeDlg::Create( wxWindow* parent, wxWindowID id, const wx
   m_pollingTimer = new wxTimer(this, YubiMixin::POLLING_TIMER_ID);
   m_pollingTimer->Start(2*YubiMixin::POLLING_INTERVAL); // 2 controls
 #endif
-  return true;
+}
+
+
+SafeCombinationChangeDlg* SafeCombinationChangeDlg::Create(wxWindow *parent, PWScore &core,
+  wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style)
+{
+  return new SafeCombinationChangeDlg(parent, core, id, caption, pos, size, style);
 }
 
 /*!
@@ -119,24 +116,6 @@ SafeCombinationChangeDlg::~SafeCombinationChangeDlg()
 #endif
 }
 
-/*!
- * Member initialisation
- */
-
-void SafeCombinationChangeDlg::Init()
-{
-////@begin SafeCombinationChangeDlg member initialisation
-  m_oldPasswdEntry = nullptr;
-  m_newPasswdEntry = nullptr;
-  m_confirmEntry = nullptr;
-#ifndef NO_YUBI
-  m_YubiBtn = nullptr;
-  m_YubiBtn2 = nullptr;
-  m_yubiStatusCtrl = nullptr;
-#endif
-////@end SafeCombinationChangeDlg member initialisation
-}
-
 /*!
  * Control creation for SafeCombinationChangeDlg
  */
@@ -197,12 +176,18 @@ void SafeCombinationChangeDlg::CreateControls()
 #endif
 
   itemFlexGridSizer4->AddStretchSpacer(0);
-  auto showCombinationCheckBox = new wxCheckBox(this, wxID_ANY, _("Show Combination"), wxDefaultPosition, wxDefaultSize, 0 );
+  auto showCombinationCheckBox = new wxCheckBox(this, ID_SHOWCOMBINATION, _("Show Combination"), wxDefaultPosition, wxDefaultSize, 0 );
   showCombinationCheckBox->SetValue(false);
+  m_isPasswordHidden = true;
   showCombinationCheckBox->Bind(wxEVT_CHECKBOX, [&](wxCommandEvent& event) {
     m_oldPasswdEntry->SecureTextfield(!event.IsChecked());
     m_newPasswdEntry->SecureTextfield(!event.IsChecked());
     m_confirmEntry->DisableAndClear(event.IsChecked());
+    m_isPasswordHidden = !event.IsChecked();
+    if(event.IsChecked())
+      m_confirmEntry->AllowEmptyCombinationOnce();
+    else
+      m_confirmEntry->ForbidEmptyCombinationOnce();
   });
   itemFlexGridSizer4->Add(showCombinationCheckBox, 1, wxALIGN_LEFT|wxALL|wxEXPAND, 5);
 
@@ -295,7 +280,7 @@ void SafeCombinationChangeDlg::OnOkClick(wxCommandEvent& WXUNUSED(evt))
       wxMessageDialog err(this, _("Cannot verify old safe combination - file gone?"),
                           _("Error"), wxOK | wxICON_EXCLAMATION);
       err.ShowModal();
-    } else if (m_confirm != m_newpasswd) {
+    } else if (m_isPasswordHidden && (m_confirm != m_newpasswd)) {
       wxMessageDialog err(this, _("New safe combination and confirmation do not match"),
                           _("Error"), wxOK | wxICON_EXCLAMATION);
       err.ShowModal();
@@ -414,6 +399,7 @@ void SafeCombinationChangeDlg::OnYubibtn2Click(wxCommandEvent& WXUNUSED(event))
     bool oldYubiChallenge = ::wxGetKeyState(WXK_SHIFT); // for pre-0.94 databases
     if (m_yubiMixin2.PerformChallengeResponse(this, m_newpasswd, response, oldYubiChallenge)) {
       m_newpasswd = response;
+      m_IsYubiProtected = true; // Don't erase YubiSK, since database is protected by Yubikey
       EndModal(wxID_OK);
     }
   }
